// Generated by CoffeeScript 1.3.3
(function() {
  'use strict';

  var BrunchTestRunner, Mocha, async, fs, fs_utils, helpers, loadJsdom, os, showJsdomNote, sysPath, test, watch,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice;

  os = require('os');

  fs = require('fs');

  sysPath = require('path');

  async = require('async');

  Mocha = require('mocha');

  fs_utils = require('../fs_utils');

  helpers = require('../helpers');

  watch = require('./watch');

  showJsdomNote = function() {
    var osMessage;
    osMessage = function() {
      if (os.platform() === 'win32') {
        return "\nBefore installing jsdom, you have to install the following dependencies:\n* Python 2.7\n* Microsoft Visual Studio or Visual C++ Express\n\nOnce you have installed the dependencies, enter the following in your terminal (in the current directory):";
      } else {
        return "In order to run tests in a CLI/jsdom environment, you have to install jsdom.";
      }
    };
    return console.log("\n\n" + (osMessage()) + "\n\na) Install jsdom for all system packages (recommended):\n  * Execute `npm install -g jsdom`\n  * Add the parent dir of jsdom to NODE_PATH environment var,\n  like `export NODE_PATH=/usr/local/lib/node_modules`\nb) Install jsdom locally for this project:\n  * Execute `npm install jsdom`");
  };

  loadJsdom = function() {
    try {
      return require('jsdom');
    } catch (error) {
      showJsdomNote();
      return process.exit(1);
    }
  };

  BrunchTestRunner = (function() {

    function BrunchTestRunner(config, options) {
      this.config = config;
      this.options = options;
      this.startTestRunner = __bind(this.startTestRunner, this);

      this.startMocha = __bind(this.startMocha, this);

      this.setupJsDom = __bind(this.setupJsDom, this);

      this.readTestFiles = __bind(this.readTestFiles, this);

      this.testFiles = helpers.findTestFiles(this.config);
      if (this.testFiles.length > 0) {
        this.setupJsDom(this.startTestRunner);
      } else {
        throw new Error("Can't find tests for this project.");
      }
    }

    BrunchTestRunner.prototype.readTestFiles = function(callback) {
      var files, getPublicPath,
        _this = this;
      getPublicPath = function() {
        var subPaths;
        subPaths = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
        return sysPath.join.apply(sysPath, [_this.config.paths["public"]].concat(__slice.call(subPaths)));
      };
      files = [getPublicPath('index.html'), getPublicPath('javascripts', 'vendor.js'), getPublicPath('javascripts', 'app.js')];
      return async.map(files, fs.readFile, callback);
    };

    BrunchTestRunner.prototype.setupJsDom = function(callback) {
      return this.readTestFiles(function(error, files) {
        var appjs, html, vendorjs;
        if (error != null) {
          throw error;
        }
        html = files[0], vendorjs = files[1], appjs = files[2];
        return loadJsdom().env({
          html: html.toString(),
          src: [vendorjs.toString(), appjs.toString()],
          done: function(error, window) {
            if (error != null) {
              throw error;
            }
            return callback(window);
          }
        });
      });
    };

    BrunchTestRunner.prototype.startMocha = function(globals) {
      var mocha, _ref, _ref1, _ref2,
        _this = this;
      helpers.extend(global, globals);
      mocha = new Mocha();
      mocha.reporter(this.options.reporter || ((_ref2 = this.config.test) != null ? _ref2.reporter : void 0) || 'spec').ui((_ref = (_ref1 = this.config.test) != null ? _ref1.ui : void 0) != null ? _ref : 'bdd');
      this.testFiles.forEach(function(file) {
        return mocha.addFile(file);
      });
      return mocha.run(function(failures) {
        return process.exit((failures > 0 ? 1 : 0));
      });
    };

    BrunchTestRunner.prototype.startTestRunner = function(window) {
      var getTestHelpersPath, testHelpersFiles,
        _this = this;
      getTestHelpersPath = function(filename) {
        return sysPath.resolve(sysPath.join(_this.config.paths.test, filename));
      };
      testHelpersFiles = [getTestHelpersPath('test-helpers.coffee', getTestHelpersPath('test-helpers.js'))];
      return async.detect(testHelpersFiles, fs_utils.exists, function(testHelpersFile) {
        var globals;
        globals = testHelpersFile != null ? require(testHelpersFile) : {};
        globals.window = window;
        return _this.startMocha(globals);
      });
    };

    return BrunchTestRunner;

  })();

  module.exports = test = function(options) {
    var watcher;
    return watcher = watch(true, options, function() {
      return new BrunchTestRunner(watcher.config, options);
    });
  };

}).call(this);
