// Generated by CoffeeScript 1.3.3
(function() {
  'use strict';

  var cachedTestFiles, callFunctionOrPass, coffeescript, deepFreeze, ensureArray, exec, express, extend, fs, getTestFiles, install, logger, recursiveExtend, replaceSlashes, setConfigDefaults, sortAlphabetically, sortByAfter, sortByBefore, sortByVendor, startDefaultServer, startsWith, sysPath;

  exec = require('child_process').exec;

  coffeescript = require('coffee-script');

  express = require('express');

  fs = require('fs');

  sysPath = require('path');

  logger = require('./logger');

  exports.startsWith = startsWith = function(string, substring) {
    return string.indexOf(substring) === 0;
  };

  exports.callFunctionOrPass = callFunctionOrPass = function(thing) {
    if (typeof thing === 'function') {
      return thing();
    } else {
      return thing;
    }
  };

  exports.ensureArray = ensureArray = function(object) {
    if (Array.isArray(object)) {
      return object;
    } else {
      return [object];
    }
  };

  exports.extend = extend = function(object, properties) {
    Object.keys(properties).forEach(function(key) {
      return object[key] = properties[key];
    });
    return object;
  };

  recursiveExtend = function(object, properties) {
    Object.keys(properties).forEach(function(key) {
      var value;
      value = properties[key];
      if (typeof value === 'object' && (value != null)) {
        return recursiveExtend(object[key], properties[key]);
      } else {
        return object[key] = properties[key];
      }
    });
    return object;
  };

  exports.deepFreeze = deepFreeze = function(object) {
    Object.keys(Object.freeze(object)).map(function(key) {
      return object[key];
    }).filter(function(value) {
      return typeof value === 'object' && (value != null) && !Object.isFrozen(value);
    }).forEach(deepFreeze);
    return object;
  };

  sortAlphabetically = function(a, b) {
    if (a < b) {
      return -1;
    } else if (a > b) {
      return 1;
    } else {
      return 0;
    }
  };

  sortByVendor = function(config, a, b) {
    var aIsVendor, bIsVendor, vendor;
    vendor = config.vendorPaths.slice().sort(sortAlphabetically);
    aIsVendor = vendor.some(function(path) {
      return exports.startsWith(a, path);
    });
    bIsVendor = vendor.some(function(path) {
      return exports.startsWith(b, path);
    });
    if (aIsVendor && !bIsVendor) {
      return -1;
    } else if (!aIsVendor && bIsVendor) {
      return 1;
    } else {
      return sortAlphabetically(a, b);
    }
  };

  sortByAfter = function(config, a, b) {
    var hasA, hasB, indexOfA, indexOfB, _ref;
    indexOfA = config.after.indexOf(a);
    indexOfB = config.after.indexOf(b);
    _ref = [indexOfA !== -1, indexOfB !== -1], hasA = _ref[0], hasB = _ref[1];
    if (hasA && !hasB) {
      return 1;
    } else if (!hasA && hasB) {
      return -1;
    } else if (hasA && hasB) {
      return indexOfA - indexOfB;
    } else {
      return sortByVendor(config, a, b);
    }
  };

  sortByBefore = function(config, a, b) {
    var hasA, hasB, indexOfA, indexOfB, _ref;
    indexOfA = config.before.indexOf(a);
    indexOfB = config.before.indexOf(b);
    _ref = [indexOfA !== -1, indexOfB !== -1], hasA = _ref[0], hasB = _ref[1];
    if (hasA && !hasB) {
      return -1;
    } else if (!hasA && hasB) {
      return 1;
    } else if (hasA && hasB) {
      return indexOfA - indexOfB;
    } else {
      return sortByAfter(config, a, b);
    }
  };

  exports.sortByConfig = function(files, config) {
    var cfg, _ref, _ref1, _ref2;
    if (toString.call(config) === '[object Object]') {
      cfg = {
        before: (_ref = config.before) != null ? _ref : [],
        after: (_ref1 = config.after) != null ? _ref1 : [],
        vendorPaths: (_ref2 = config.vendorPaths) != null ? _ref2 : []
      };
      return files.slice().sort(function(a, b) {
        return sortByBefore(cfg, a, b);
      });
    } else {
      return files;
    }
  };

  exports.install = install = function(rootPath, callback) {
    var prevDir;
    if (callback == null) {
      callback = (function() {});
    }
    prevDir = process.cwd();
    logger.info('Installing packages...');
    process.chdir(rootPath);
    return exec('npm install', function(error, stdout, stderr) {
      var log;
      process.chdir(prevDir);
      if (error != null) {
        log = stderr.toString();
        logger.error(log);
        return callback(log);
      }
      return callback(null, stdout);
    });
  };

  startDefaultServer = function(port, path, base, callback) {
    var server;
    server = express.createServer();
    server.use(function(request, response, next) {
      response.header('Cache-Control', 'no-cache');
      return next();
    });
    server.use(base, express["static"](path));
    server.all("" + base + "/*", function(request, response) {
      return response.sendfile(sysPath.join(path, 'index.html'));
    });
    server.listen(parseInt(port, 10));
    server.on('listening', callback);
    return server;
  };

  exports.startServer = function(config, callback) {
    var onListening, server;
    if (callback == null) {
      callback = (function() {});
    }
    onListening = function() {
      logger.info("application started on http://localhost:" + config.server.port + "/");
      return callback();
    };
    if (config.server.path) {
      try {
        server = require(sysPath.resolve(config.server.path));
        return server.startServer(config.server.port, config.paths["public"], onListening);
      } catch (error) {
        return logger.error("couldn\'t load server " + config.server.path + ": " + error);
      }
    } else {
      return startDefaultServer(config.server.port, config.paths["public"], config.server.base, onListening);
    }
  };

  exports.replaceSlashes = replaceSlashes = function(config) {
    var changePath, files;
    changePath = function(string) {
      return string.replace(/\//g, '\\');
    };
    files = config.files || {};
    Object.keys(files).forEach(function(language) {
      var lang, newJoinTo, order;
      lang = files[language] || {};
      order = lang.order || {};
      Object.keys(order).forEach(function(orderKey) {
        return lang.order[orderKey] = lang.order[orderKey].map(changePath);
      });
      switch (toString.call(lang.joinTo)) {
        case '[object String]':
          return lang.joinTo = changePath(lang.joinTo);
        case '[object Object]':
          newJoinTo = {};
          Object.keys(lang.joinTo).forEach(function(joinToKey) {
            return newJoinTo[changePath(joinToKey)] = lang.joinTo[joinToKey];
          });
          return lang.joinTo = newJoinTo;
      }
    });
    return config;
  };

  exports.setConfigDefaults = setConfigDefaults = function(config, configPath) {
    var apps, join, joinRoot, paths, _base, _base1, _base2, _ref, _ref1, _ref10, _ref11, _ref12, _ref13, _ref14, _ref2, _ref3, _ref4, _ref5, _ref6, _ref7, _ref8, _ref9,
      _this = this;
    join = function(parent, name) {
      return sysPath.join(config.paths[parent], name);
    };
    joinRoot = function(name) {
      return join('root', name);
    };
    paths = (_ref = config.paths) != null ? _ref : config.paths = {};
    if ((_ref1 = paths.root) == null) {
      paths.root = (_ref2 = config.rootPath) != null ? _ref2 : '.';
    }
    if ((_ref3 = paths["public"]) == null) {
      paths["public"] = (_ref4 = config.buildPath) != null ? _ref4 : joinRoot('public');
    }
    apps = ['app', 'test', 'vendor'].map(joinRoot);
    if ((_ref5 = paths.app) == null) {
      paths.app = apps[0];
    }
    if ((_ref6 = paths.test) == null) {
      paths.test = apps[1];
    }
    if ((_ref7 = paths.vendor) == null) {
      paths.vendor = [apps[2], join('test', 'vendor')];
    }
    if ((_ref8 = paths.assets) == null) {
      paths.assets = [join('app', 'assets'), join('test', 'assets')];
    }
    if ((_ref9 = paths.ignored) == null) {
      paths.ignored = function(path) {
        return startsWith(sysPath.basename(path), '_');
      };
    }
    paths.config = configPath != null ? configPath : joinRoot('config');
    if ((_ref10 = paths.packageConfig) == null) {
      paths.packageConfig = joinRoot('package.json');
    }
    if ((_ref11 = config.server) == null) {
      config.server = {};
    }
    if ((_ref12 = (_base = config.server).base) == null) {
      _base.base = '';
    }
    if ((_ref13 = (_base1 = config.server).port) == null) {
      _base1.port = 3333;
    }
    if ((_ref14 = (_base2 = config.server).run) == null) {
      _base2.run = false;
    }
    config.paths.assets = ensureArray(config.paths.assets);
    config.paths.vendor = ensureArray(config.paths.vendor);
    if (process.platform === 'win32') {
      replaceSlashes(config);
    }
    return config;
  };

  exports.loadConfig = function(configPath, options) {
    var config, fullPath;
    if (configPath == null) {
      configPath = 'config';
    }
    if (options == null) {
      options = {};
    }
    fullPath = sysPath.resolve(configPath);
    delete require.cache[fullPath];
    try {
      config = require(fullPath).config;
    } catch (error) {
      throw new Error("couldn\'t load config " + configPath + ". " + error);
    }
    setConfigDefaults(config, fullPath);
    recursiveExtend(config, options);
    deepFreeze(config);
    return config;
  };

  exports.loadPackages = function(rootPath, callback) {
    var nodeModules;
    rootPath = sysPath.resolve(rootPath);
    nodeModules = "" + rootPath + "/node_modules";
    return fs.readFile(sysPath.join(rootPath, 'package.json'), function(error, data) {
      var deps, json, plugins, _ref;
      if (error != null) {
        return callback(error);
      }
      json = JSON.parse(data);
      deps = Object.keys(extend((_ref = json.devDependencies) != null ? _ref : {}, json.dependencies));
      try {
        plugins = deps.map(function(dependency) {
          return require("" + nodeModules + "/" + dependency);
        });
      } catch (err) {
        error = err;
      }
      return callback(error, plugins);
    });
  };

  exports.getPlugins = function(packages, config) {
    return packages.filter(function(plugin) {
      return (plugin.prototype != null) && plugin.prototype.brunchPlugin;
    }).map(function(plugin) {
      return new plugin(config);
    });
  };

  getTestFiles = function(config) {
    var files, isTestFile, joinPublic, joinTo;
    isTestFile = function(generatedFile) {
      return exports.startsWith(generatedFile, sysPath.normalize('test/')) && generatedFile.lastIndexOf('vendor') === -1;
    };
    joinPublic = function(generatedFile) {
      return sysPath.join(config.paths["public"], generatedFile);
    };
    joinTo = config.files.javascripts.joinTo;
    files = typeof joinTo === 'string' ? [joinTo] : Object.keys(joinTo);
    return files.filter(isTestFile).map(joinPublic);
  };

  cachedTestFiles = null;

  exports.findTestFiles = function(config) {
    return cachedTestFiles != null ? cachedTestFiles : cachedTestFiles = getTestFiles(config);
  };

}).call(this);
